/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: WoUnmarshallerFactory.java,v bc38d0706915 2010/07/12 02:16:20 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.factsheet.bbc;

import javax.annotation.Nonnull;
import java.util.List;
import org.openrdf.model.Statement;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.semantic.io.spi.StatementUnmarshaller;
import it.tidalwave.semantic.io.spi.StatementUnmarshallerFactory;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public class WoUnmarshallerFactory implements StatementUnmarshallerFactory
  {
    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Nonnull
    public StatementUnmarshaller findStatementUnmarshallerFor (final @Nonnull List<Statement> statements)
      throws NotFoundException
      {
        for (final Statement statement : statements)
          {
            final String subject = statement.getSubject().stringValue();
            final String object = statement.getObject().stringValue();
            final String predicate = statement.getPredicate().stringValue();

            if (predicate.equals(WoVocabulary.RDF_TYPE.stringValue()))
              {
                if (object.equals(WoVocabulary.WO_SPECIES.stringValue()))
                  {
                    return new FactSheetUnmarshaller();
                  }

                if (object.equals(WoVocabulary.PO_CLIP.stringValue()))
                  {
                    return new MovieUnmarshaller();
                  }

                if (object.equals(WoVocabulary.PO_EPISODE.stringValue()))
                  {
                    return new MovieUnmarshaller();
                  }

                if (object.equals(WoVocabulary.FOAF_IMAGE.stringValue()))
                  {
                    return new StillImageUnmarshaller();
                  }
              }

            if (subject.startsWith("http://www.bbc.co.uk/nature/adaptations/"))
              {
                return new AdaptationUnmarshaller();
              }

            if (subject.startsWith("http://www.bbc.co.uk/nature/habitats/"))
              {
                return new HabitatUnmarshaller();
              }

            if (predicate.equals(WoVocabulary.FOAF_PRIMARY_TOPIC.stringValue()))
              {
                return new BiblioUnmarshaller();
              }
          }

        throw new NotFoundException();
      }
  }
